import json
from functools import partial

import geopandas as gpd
import pandas as pd
import requests
from shapely.geometry import mapping


class TSService:
    """This class provides a Python API for the PROBA-V MEP Time Series Service. https://proba-v-mep.esa.int/api/timeseries/apidocs/"""
    def __init__(self):
        pass

    def timeseries(self, layername,geodataframe,startdate,enddate) -> pd.DataFrame:
        """Retrieves all timeseries corresponding to features in a GeoPandas dataframe.

        """
        #reproject to lat lon, which is what our web service uses
        parcels_latlon_df = geodataframe.to_crs(epsg=4326)
        print(parcels_latlon_df.geometry)
        #after transformation, there appear to be some empty polygons, we filter them out
        parcels_latlon_df = parcels_latlon_df[~parcels_latlon_df.geometry.is_empty]
        #convert geometry to json
        parcels_latlon_df['geometry_json'] = parcels_latlon_df.geometry.apply(lambda g:json.dumps(mapping(g)))

        return self.retrieve_curves(layername,parcels_latlon_df,pd.to_datetime(startdate),pd.to_datetime(enddate))

    @classmethod
    def retrieve_curves(cls,  layername,geodataframe,startdate,enddate):
        if 'progress_apply' in dir(geodataframe):
            curves = geodataframe.progress_apply(partial(cls._retrieve_curve,layername,startdate,enddate), axis=1)
        else:
            curves = geodataframe.apply(partial(cls._retrieve_curve,layername,startdate,enddate), axis=1)

        #print(curves.columns)
        return curves

    @classmethod
    def _retrieve_curve(cls,layername,startdate,enddate,field_data:gpd.GeoDataFrame):

        shape = field_data['geometry_json']

        #url = "http://192.168.205.137:8080/v1.0/ts/"+layername+"/geometry?endDate="+enddate.date().isoformat()+"&startDate="+startdate.date().isoformat()
        url = "http://192.168.205.137:8080/v1.0/ts/"+layername+"/point?endDate="+enddate.date().isoformat()+"&startDate="+startdate.date().isoformat() + "&lat=" + str(field_data.geometry.centroid.y) + "&lon=" + str(field_data.geometry.centroid.x)
        print(url)
        #curve = requests.post(url,data=shape)
        curve = requests.get(url)
        print(curve.text)
        if(curve.status_code == 200):
            try:
                if(len(curve.json()['results']) > 0):
                    date = pd.to_datetime([x['date'] for x in curve.json()['results']])
                    values = pd.to_numeric([x['result']['average'] for x in curve.json()['results']],errors='coerce')

                    series = pd.Series(values,index=date,name=field_data.name)
                    print(series)
                    return  series
                else:
                    print("Empty series for field: "+field_data.name)
                    return None
            except ValueError:
                print("Invalid json")
                print(curve.text)
                return None
        else:
            print(curve.text)
            print("Error for: "+str(field_data))
            return None